#!/usr/bin/env python
#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#  - Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#  - Neither the name of Arista Networks nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import asyncore
import imp
import json
import re
import os
import random
import subprocess
import string                        #pylint: disable=W0402
import shutil
import smtpd
import time
import thread
import unittest

import BaseHTTPServer

from collections import namedtuple


#pylint: disable=C0103
Response = namedtuple('Response', 'content_type status contents headers')
#pylint: enable=C0103

RC_EOS = 'rc.eos'
BOOT_EXTENSIONS = 'boot-extensions'
BOOT_EXTENSIONS_FOLDER = '.extensions'
STARTUP_CONFIG = 'startup-config'

ZTPS_SERVER = '127.0.0.1'
ZTPS_PORT = 12345

EAPI_SERVER = '127.0.0.1'
EAPI_PORT = 1080

SMTP_SERVER = '127.0.0.1'
SMTP_PORT = 2525

BOOTSTRAP_FILE = 'client/bootstrap'

CLI_LOG = '/tmp/FastCli-log'
EAPI_LOG = '/tmp/eapi-log-%s' % os.getpid()

STATUS_OK = 200
STATUS_CREATED = 201
STATUS_BAD_REQUEST = 400
STATUS_NOT_FOUND = 404
STATUS_CONFLICT = 409

SYSTEM_MAC = '1234567890'


def raise_exception(exception):
    #pylint: disable=C0301, C0321

    # Uncomment the following line for debugging
    # import pdb; pdb.set_trace()

    raise Exception('%s\nUncomment line in client_test_lib.py:raise_'
                    'exception for debugging' % exception.message)

ztps = None    #pylint: disable=C0103
def start_ztp_server():
    global ztps     #pylint: disable=W0603
    if not ztps:
        ztps = ZTPServer()
        ztps.start()
    else:
        ztps.cleanup()
    return ztps

smtp = None    #pylint: disable=C0103
def start_smtp_server():
    global smtp     #pylint: disable=W0603
    if not smtp:
        smtp = SmtpServer()
        smtp.start()
    return smtp

eapis = None    #pylint: disable=C0103
def start_eapi_server():
    global eapis    #pylint: disable=W0603
    if not eapis:
        eapis = EAPIServer()
        eapis.start()
    else:
        eapis.cleanup()
    return eapis

def remove_file(filename):
    try:
        os.remove(filename)
    except OSError:
        pass

def clear_cli_log():
    remove_file(CLI_LOG)

def clear_eapi_log():
    remove_file(EAPI_LOG)

def clear_logs():
    clear_cli_log()
    clear_eapi_log()

def eapi_log():
    try:
        return [x.strip()
                for x in open(EAPI_LOG, 'r').readlines()]
    except IOError:
        return []

def cli_log():
    try:
        return [x.strip().split('-c ')[ -1 ]
                for x in open(CLI_LOG, 'r').readlines()]
    except IOError:
        return []

def file_log(filename, ignore_string=None):

    try:
        lines = [x.strip() for x in open(filename, 'r').readlines()]
        if ignore_string:
            return [y for y in lines if y and ignore_string not in y]
        else:
            return [y for y in lines if y]
    except IOError:
        return []

def get_action(action):
    return open('actions/%s' % action, 'r').read()

def startup_config_action(lines=None):
    startup_config = '/tmp/ztps-flash-%s/startup-config' % os.getpid()
    if not lines:
        lines = ['test']

    user = os.getenv('USER')
    return '''#!/usr/bin/env python
import os
import pwd

def main(attributes):
   user = pwd.getpwnam('%s').pw_uid
   group = pwd.getpwnam('%s').pw_gid

   f = file('%s', 'w')
   f.write(\'\'\'%s\'\'\')
   f.close()

   os.chmod('%s', 0777)
   os.chown('%s', user, group)
''' % (user, user, startup_config, '\n'.join(lines),
       startup_config, startup_config)

def print_action(msg='TEST', use_attribute=False, create_copy=False):
    #pylint: disable=E0602
    if use_attribute and create_copy:
        return '''#!/usr/bin/env python

def main(attributes):
   attrs = attributes.copy()
   print attrs.get('print_action-attr')
'''

    if use_attribute:
        return '''#!/usr/bin/env python

def main(attributes):
   print attributes.get('print_action-attr')
'''
    else:
        return '''#!/usr/bin/env python

def main(attributes):
   print '%s'
''' % msg

def print_attributes_action(attributes):
    #pylint: disable=E0602
    result = '''#!/usr/bin/env python

def main(attributes):
'''
    for attr in attributes:
        result += '    print attributes.get(\'%s\')\n' % attr
    return result

def fail_flash_file_action(flash, filename):
    '''Creates file on flash and then fails'''

    return '''#!/usr/bin/env python

def main(attributes):
   open('%s/%s', 'w').write('test')
   raise Exception('Ops! I failed! :(')
''' % (flash, filename)

def fail_action():
    return '''#!/usr/bin/env python

def main(attributes):
   raise Exception('Ops! I failed! :(')
'''

def erroneous_action():
    return '''THIS_IS_NOT_PYTHON'''

def missing_main_action():
    return '''#!/usr/bin/env python'''

def wrong_signature_action():
    return '''#!/usr/bin/env python

def main():
   pass
'''

def exception_action():
    return '''#!/usr/bin/env python

def main(attributes):
   raise Exception
'''

def random_string():
    return ''.join(random.choice(
            string.ascii_uppercase +
            string.digits) for _ in range(random.randint(3, 20)))


class Bootstrap(object):
    #pylint: disable=R0201

    def __init__(self, server=None, eapi_port=None,
                 ztps_default_config=False):
        os.environ['PATH'] += ':%s/test/client' % os.getcwd()

        self.server = server if server else '%s:%s' % (ZTPS_SERVER, ZTPS_PORT)
        self.eapi_port = eapi_port if eapi_port else EAPI_PORT

        self.output = None
        self.error = None
        self.return_code = None
        self.filename = None
        self.module = None

        self.eapi = start_eapi_server()
        self.ztps = start_ztp_server()
        self.smtp = start_smtp_server()

        self.flash = '/tmp/ztps-flash-%s' % os.getpid()
        self.temp = '/tmp/ztps-tmp-%s' % os.getpid()

        self.rc_eos = '%s/%s' % (self.flash, RC_EOS)
        self.boot_extensions = '%s/%s' % (self.flash, BOOT_EXTENSIONS)
        self.boot_extensions_folder = '%s/%s' % (self.flash, 
                                          BOOT_EXTENSIONS_FOLDER)
        self.startup_config = '%s/%s' % (self.flash, STARTUP_CONFIG)

        self.configure()

        if ztps_default_config:
            self.ztps.set_config_response()
            self.ztps.set_node_check_response()

    def configure(self):
        for folder in [self.flash, self.temp]:
            if not os.path.exists(folder):
                os.makedirs(folder)

        infile = open(BOOTSTRAP_FILE)
        self.filename = '/tmp/bootstrap-%s' % os.getpid()
        outfile = open(self.filename, 'w')

        for line in infile:
            line = line.replace('$SERVER', 'http://%s' % self.server)
            line = line.replace("COMMAND_API_SERVER = 'localhost'",
                                "COMMAND_API_SERVER = 'localhost:%s'" %
                                self.eapi_port)

            line = line.replace("FLASH = '/mnt/flash'",
                                "FLASH = '%s'" % self.flash)

            line = line.replace("TEMP = '/tmp'", "TEMP = '%s'" % 
                                self.temp)

           # Reduce HTTP timeout
            if re.match('^HTTP_TIMEOUT', line):
                line = 'HTTP_TIMEOUT = 0.01'

            outfile.write(line)

        infile.close()
        outfile.close()

        os.chmod(self.filename, 0777)
        self.module = imp.load_source('bootstrap', self.filename)

    def end_test(self):
        shutil.rmtree(self.temp)
        shutil.rmtree(self.flash)

        # Clean up actions
        for url in self.ztps.responses.keys():
            filename = url.split('/')[-1]
            remove_file('/tmp/%s' % filename)
            remove_file('/tmp/%sc' % filename)

        # Clean up log files
        for filename in os.listdir('/tmp'):
            if re.search('^ztps-log-', filename):
                os.remove(os.path.join('/tmp', filename))

        # Clean up bootstrap script
        remove_file(self.filename)
        remove_file('%sc' % self.filename)

        # Clean up logs
        clear_logs()

    def start_test(self):
        try:
            proc = subprocess.Popen([self.filename],
                                    stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
            (self.output, self.error) = proc.communicate()
        finally:
            os.remove(self.filename)

        self.return_code = proc.returncode         #pylint: disable=E1101

    def node_information_collected(self):
        cmds = ['show version',          # Collect system MAC for logging
                'show lldp neighbors']
        return [x for x in eapi_log() if x != 'enable'][-2:] == cmds

    def eapi_configured(self):
        cmds = ['configure',
                'username ztps secret ztps-password privilege 15',
                'management api http-commands',
                'no protocol https',
                'protocol http',
                'no shutdown']
        return cli_log()[:6] == cmds

    def eapi_node_information_collected(self):
        return self.eapi_configured() and self.node_information_collected()

    def server_connection_failure(self):
        return ('server connection error' in self.output or
                'Read timed out' in self.output) and \
               self.return_code

    def eapi_failure(self):
        return 'unable to enable eAPI' in self.output and \
               self.return_code

    def unexpected_response_failure(self):
        return 'unexpected reponse from server' in self.output and \
               self.return_code

    def node_not_found_failure(self):
        return 'node not found on server' in self.output and \
               self.return_code

    def toplogy_check_failure(self):
        return 'server-side topology check failed' in self.output and \
               self.return_code

    def action_not_found_failure(self):
        return 'action not found on server' in self.output and \
               self.return_code

    def missing_startup_config_failure(self):
        return 'startup configuration is missing' in self.output and \
               self.return_code

    def action_failure(self):
        return 'executing action failed' in self.output and \
               self.return_code

    def invalid_definition_format(self):
        return 'section missing from definition' in self.output and \
               self.return_code

    def invalid_definition_location_failure(self):
        return 'invalid definition location received ' \
               'from server' in self.output and \
               self.return_code

    def success(self):
        return 'ZTP bootstrap completed successfully!' in self.output and \
               not self.return_code


class EAPIServer(object):
    #pylint: disable=C0103,E0213,W0201

    def __init__(self, mac=SYSTEM_MAC, model='',
                 serial_number='', version=''):
        self.mac = mac
        self.model = model
        self.serial_number = serial_number
        self.version = version

        self.fail_commands = []

    def cleanup(self):
        self.responses = {}

    def add_failing_command(self, cmd):
        self.fail_commands += [cmd]

    def start(self):
        thread.start_new_thread(self._run, ())

    def _run(self):

        class EAPIHandler(BaseHTTPServer.BaseHTTPRequestHandler):

            def do_POST(req):
                request = req.rfile.read(int(req.headers.getheader(
                            'content-length')))
                cmds = [x for x in json.loads(request)['params'][1] if x]
                if cmds:
                    open(EAPI_LOG, 'a+b').write('%s\n' % '\n'.join(cmds))

                print 'EAPIServer: responding to request:%s (%s)' % (
                    req.path, ', '.join(cmds))

                if [x for x in cmds if x in self.fail_commands]:
                    print 'EAPIServer: failed on-demand'
                    req.send_response(STATUS_BAD_REQUEST)                    
                    return

                req.send_response(STATUS_OK)

                if req.path == '/command-api':
                    req.send_header('Content-type', 'application/json')
                    req.end_headers()
                    if cmds == ['enable', 'show version']:
                        req.wfile.write(json.dumps(
                                {'result' :
                                 [{},
                                  {'modelName' : self.model,
                                   'version' : self.version,
                                   'serialNumber' : self.serial_number,
                                   'systemMacAddress' : self.mac}]}))
                    elif cmds == ['enable', 'show lldp neighbors']:
                        req.wfile.write(json.dumps({'result' :
                                                  [{},
                                                   {'lldpNeighbors': []}]}))
                    else:
                        req.wfile.write(json.dumps({'result' : [{}]}))
                    print 'EAPIServer: RESPONSE: [{}]'
                else:
                    print 'EAPIServer: No RESPONSE'

        server_class = BaseHTTPServer.HTTPServer
        httpd = server_class((EAPI_SERVER, EAPI_PORT), EAPIHandler)
        print time.asctime(), 'EAPIServer: Server starts - %s:%s' % (
            EAPI_SERVER, EAPI_PORT)

        try:
            httpd.serve_forever()
        except KeyboardInterrupt:
            pass
        finally:
            httpd.server_close()
            print time.asctime(), 'EAPIServer: Server stops - %s:%s' % (
                EAPI_SERVER, EAPI_PORT)


class ZTPServer(object):
    #pylint: disable=C0103,,E0213

    # { <URL>: ( <CONTNENT-TYPE>, <STATUS>, <RESPONSE> ) }
    responses = {}

    def cleanup(self):
        self.responses = {}

    def set_file_response(self, filename, output,
                          content_type='text/plain',
                          status=STATUS_OK):
        self.responses['/%s' % filename ] = Response(
            content_type, status,
            output, {})

        meta = { 'size': len( output ) }
        self.responses['/meta/%s' % filename ] = Response(
            'application/json', 200,
            json.dumps(meta), {})

    def set_action_response(self, action, output,
                            content_type='text/x-python',
                            status=STATUS_OK):
        self.responses['/actions/%s' % action ] = Response(
            content_type, status,
            output, {})

    def set_config_response(self, logging=None, xmpp=None,
                            content_type='application/json',
                            status=STATUS_OK):
        response = { 'logging': [],
                     'xmpp': {}
                     }
        if logging:
            response['logging'] = logging

        if xmpp:
            response['xmpp'] = xmpp

        self.responses['/bootstrap/config'] = Response(
            content_type, status,
            json.dumps(response), {})

    def set_node_check_response(self, content_type='text/html',
                                status=None, location=None):
        if status is None:
            status = random.choice([STATUS_CONFLICT,
                                    STATUS_CREATED])

        headers = {}
        if location:
            headers['location'] = location

        self.responses['/nodes'] = Response(
            content_type, status, 
            '', headers)

    def set_bogus_definition_response(self):
        self.responses['/nodes/%s' % SYSTEM_MAC] = Response(
            'application/json', STATUS_OK,
            json.dumps({}), {})

    def set_definition_response(self, node_id=SYSTEM_MAC,
                                name='DEFAULT_DEFINITION',
                                actions=None,
                                content_type='application/json',
                                status=STATUS_OK):
        response = { 'name': name,
                     'actions': [],
                     }
        if actions:
            response['actions'] += actions

        self.responses['/nodes/%s' % node_id] = Response(
            content_type, status,
            json.dumps(response), {})

    def start(self):
        thread.start_new_thread(self._run, ())

    def _run(self):

        class ZTPSHandler(BaseHTTPServer.BaseHTTPRequestHandler):

            @classmethod
            def do_request(cls, req):
                if req.path in self.responses.keys():
                    response = self.responses[req.path]
                    req.send_response(response.status)
                    req.error_content_type = response.content_type

                    req.send_header('Content-type', response.content_type)
                    for name, value in response.headers.iteritems():
                        req.send_header(name, value)

                    req.end_headers()
                    req.wfile.write(response.contents)
                    print 'ZTPS: RESPONSE: (ct=%s, status=%s, output=%s...)' % (
                        response[0],
                        response[1],
                        response[2][:100])
                else:
                    print 'ZTPS: No RESPONSE'

            def do_GET(req):
                print 'ZTPS: responding to GET request:%s' % req.path
                ZTPSHandler.do_request(req)

            def do_POST(req):
                print 'ZTPS: responding to POST request:%s' % req.path
                headers = self.responses['/nodes'].headers
                if 'location' not in headers:
                    length = req.headers.getheader('content-length')
                    node_id = json.loads(req.rfile.read(
                            int(length)))['systemmac']
                    location  = 'http://%s:%s/nodes/%s' % (ZTPS_SERVER, 
                                                           ZTPS_PORT,
                                                           node_id)
                    self.responses['/nodes'].headers['location'] = location

                ZTPSHandler.do_request(req)

        server_class = BaseHTTPServer.HTTPServer
        httpd = server_class((ZTPS_SERVER, ZTPS_PORT), ZTPSHandler)

        print time.asctime(), 'ZTPS: Server starts - %s:%s' % (
            ZTPS_SERVER, ZTPS_PORT)
        try:
            httpd.serve_forever()
        except KeyboardInterrupt:
            pass
        finally:
            httpd.server_close()
            print time.asctime(), 'ZTPS: Server stops - %s:%s' % (
                ZTPS_SERVER, ZTPS_PORT)


class SmtpServer(object):
    #pylint: disable=E0211

    def start(self):
        thread.start_new_thread(self._run, ())

    @classmethod
    def _run(cls):

        class SMTPServer(smtpd.SMTPServer):

            def __init__(*args, **kwargs):
                print "SMTP: Running smtp server on port 2525"
                smtpd.SMTPServer.__init__(*args, **kwargs)

            def process_message(*args, **kwargs):
                pass

        smtp_server = SMTPServer((SMTP_SERVER, SMTP_PORT), None)
        print time.asctime(), 'SMTP: Server starts - %s:%s' % (
            SMTP_SERVER, SMTP_PORT)
        try:
            asyncore.loop()
        except KeyboardInterrupt:
            pass
        finally:
            smtp_server.close()
            print time.asctime(), 'SMTPS: Server stops - %s:%s' % (
                SMTP_SERVER, SMTP_PORT)

class ActionFailureTest(unittest.TestCase):
    #pylint: disable=R0904

    def basic_test(self, action, return_string, attributes=None,
                   action_value=None, file_responses=None):
        if not attributes:
            attributes = {}

        if not file_responses:
            file_responses = {}

        if not action_value:
            action_value = get_action(action)

        bootstrap = Bootstrap(ztps_default_config=True)
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'test_action',
                      'attributes' : attributes}])
        bootstrap.ztps.set_action_response('test_action',
                                           action_value)

        for key, value in file_responses.iteritems():
            bootstrap.ztps.set_file_response(key, value)            

        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.action_failure())
            msg = [x for x in bootstrap.output.split('\n') if x][-1]
            self.failUnless('%s' % return_string in msg)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()



#!/usr/bin/env python
#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#  - Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#  - Neither the name of Arista Networks nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#pylint: disable=R0904,F0401,W0232,E1101

import os
import os.path
import unittest
import shutil
import sys

sys.path.append('test/client')

from client_test_lib import Bootstrap, ActionFailureTest
from client_test_lib import file_log, get_action, random_string
from client_test_lib import startup_config_action
from client_test_lib import raise_exception

class FailureTest(ActionFailureTest):

    def test_missing_url(self):
        self.basic_test('install_cli_plugin',
                        'Missing attribute(\'url\')')

    def test_url_failure(self):
        self.basic_test('install_cli_plugin',
                        'Unable to retrieve CliPlugin from URL',
                        attributes={'url' :
                                    random_string()})


class SuccessTest(unittest.TestCase):

    def test_success(self):
        bootstrap = Bootstrap(ztps_default_config=True)
        plugin = random_string()
        url = 'http://%s/%s' % (bootstrap.server, plugin)
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'},
                     {'action' : 'test_action',
                      'attributes' : {'url' : url}}])

        bootstrap.ztps.set_action_response(
            'startup_config_action', startup_config_action())

        plugin_dir = '/tmp'
        persistent_dir = '/tmp/persistent'
        action = get_action('install_cli_plugin')
        action = action.replace('/usr/lib/python2.7/site-packages/CliPlugin',
                                plugin_dir)

        action = action.replace('/mnt/flash/.ztp-plugins',
                                persistent_dir)
        bootstrap.ztps.set_action_response('test_action',
                                           action)

        contents = random_string()
        bootstrap.ztps.set_file_response(plugin, contents)
        bootstrap.start_test()

        try:
            self.failUnless(os.path.isfile(bootstrap.rc_eos))
            log = file_log(bootstrap.rc_eos)
            self.failUnless('#!/bin/bash' in log)
            self.failUnless('sudo cp %s/%s %s' %
                            (persistent_dir, plugin, plugin_dir) in log)

            self.failUnless(contents in
                            file_log('%s/%s' % (persistent_dir, plugin)))
            self.failUnless(bootstrap.success())
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            shutil.rmtree(persistent_dir)
            bootstrap.end_test()

    def test_ztps_path_success(self):
        bootstrap = Bootstrap(ztps_default_config=True)
        plugin = url = random_string()
        ztps_server = 'http://%s' % bootstrap.server
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'},
                     {'action' : 'test_action',
                      'attributes' : {'url' : url,
                                      'ztps_server': ztps_server}}])

        bootstrap.ztps.set_action_response(
            'startup_config_action', startup_config_action())

        plugin_dir = '/tmp'
        persistent_dir = '/tmp/persistent'
        action = get_action('install_cli_plugin')
        action = action.replace('/usr/lib/python2.7/site-packages/CliPlugin',
                                plugin_dir)

        action = action.replace('/mnt/flash/.ztp-plugins',
                                persistent_dir)
        bootstrap.ztps.set_action_response('test_action',
                                           action)

        contents = random_string()
        bootstrap.ztps.set_file_response(plugin, contents)
        bootstrap.start_test()

        try:
            self.failUnless(os.path.isfile(bootstrap.rc_eos))
            log = file_log(bootstrap.rc_eos)
            self.failUnless('#!/bin/bash' in log)
            self.failUnless('sudo cp %s/%s %s' %
                            (persistent_dir, plugin, plugin_dir) in log)

            self.failUnless(contents in
                            file_log('%s/%s' % (persistent_dir, plugin)))
            self.failUnless(bootstrap.success())
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            shutil.rmtree(persistent_dir)
            bootstrap.end_test()


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import unittest
import yaml

from mock import patch, Mock

import ztpserver.serializers
import ztpserver.topology

from ztpserver.topology import Neighbordb, Pattern
from ztpserver.topology import create_node, load_file, load_neighbordb
from ztpserver.topology import neighbordb_path, replace_config_action
from ztpserver.topology import load_pattern
from server_test_lib import enable_logging, random_string

class NeighbordbUnitTests(unittest.TestCase):

    def test_neighbordb_path(self):
        result = neighbordb_path()
        self.assertEqual(result, '/usr/share/ztpserver/neighbordb')

    @patch('ztpserver.topology.load')
    def test_load_file(self, m_load):
        result = load_file(random_string(),
                           random_string(),
                           random_string())
        self.assertEqual(result, m_load.return_value)

    @patch('ztpserver.topology.validate_neighbordb')
    @patch('ztpserver.topology.load')
    def test_load_file_failure(self, m_load, _):
        m_load.side_effect = ztpserver.serializers.SerializerError
        self.assertRaises(ztpserver.serializers.SerializerError,
                          load_file,
                          random_string(),
                          random_string(),
                          random_string())

    @patch('ztpserver.topology.validate_neighbordb')
    @patch('ztpserver.topology.load')
    def test_load_neighbordb(self, _, m_load):
        contents = '''
            variables:
                foo: bar
            patterns:
                - name: dummy pattern
                  definition: dummy_definition
                  interfaces:
                    - any: any
        '''
        m_load.return_value = yaml.load(contents)
        result = load_neighbordb(random_string())
        self.assertIsNotNone(result)

    @patch('ztpserver.topology.load')
    def test_load_neighbordb_no_variables(self, m_load):
        # github issue #114
        contents = """
            patterns:
                - name: dummy pattern
                  definition: dummy_definition
                  interfaces:
                    - any: any
        """
        m_load.return_value = yaml.load(contents)
        result = load_neighbordb(random_string())
        self.assertIsInstance(result, Neighbordb)

    def test_load_pattern_minimal(self):
        pattern = load_pattern({'name': random_string(),
                                'definition': random_string(),
                                'interfaces': []})
        self.assertIsInstance(pattern, Pattern)

    def test_load_pattern_with_interfaces(self):
        # github issue #128
        contents = """
            name: test
            # Default pattern - always succeeds
            definition: dummy_definition
            interfaces:
                - any: any:any
        """
        kwargs = yaml.load(contents)
        pattern = load_pattern(kwargs)
        self.assertIsInstance(pattern, Pattern)

    def test_replace_config_action(self):
        resource = random_string()
        result = replace_config_action(resource)
        self.assertEqual('install static startup-config file', result['name'])
        self.assertEqual('replace_config', result['action'])
        self.assertTrue(result['always_execute'])

    def test_create_node_fixup_systemmac_colon(self):
        attrs = Mock(systemmac='99:99:99:99:99:99')
        result = create_node({'systemmac': 
                              attrs.systemmac})
        self.assertTrue(':' not in result.systemmac)

    def test_create_node_fixup_systemmac_period(self):
        attrs = Mock(systemmac='99.99.99.99.99.99')
        result = create_node({'systemmac': 
                              attrs.systemmac})
        self.assertTrue('.' not in result.systemmac)

if __name__ == '__main__':
    enable_logging()
    unittest.main()



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
# pylint: disable=C0103,W1201
#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import unittest

import ztpserver.types

class TestTypes(unittest.TestCase):

    def test_create_string_defaults(self):
        obj = ztpserver.types.String()
        self.assertIsInstance(obj, ztpserver.types.String)

    def test_create_string_choices(self):
        obj = ztpserver.types.String(['one', 'two'])
        self.assertIsInstance(obj, ztpserver.types.String)
        self.assertEqual(repr(obj), "String(choices=one,two)")

    def test_create_string_choices_integers(self):
        obj = ztpserver.types.String([1, 2])
        self.assertIsInstance(obj, ztpserver.types.String)
        self.assertEqual(repr(obj), "String(choices=1,2)")

    def test_call_string_using_defaults_and_string(self):
        obj = ztpserver.types.String()
        self.assertIsInstance(obj, ztpserver.types.String)
        self.assertEqual(obj('hello'), 'hello')

    def test_call_string_using_defaults_and_int(self):
        obj = ztpserver.types.String()
        self.assertIsInstance(obj, ztpserver.types.String)
        self.assertEqual(obj(1), '1')

    def test_call_string_using_defaults_and_bool(self):
        obj = ztpserver.types.String()
        self.assertIsInstance(obj, ztpserver.types.String)
        self.assertEqual(obj(True), 'True')

    def test_call_string_using_defaults_and_list(self):
        obj = ztpserver.types.String()
        self.assertIsInstance(obj, ztpserver.types.String)
        self.assertEqual(obj(['one']), "['one']")

    def test_call_string_with_choices_valid(self):
        obj = ztpserver.types.String(['one', 'two'])
        self.assertIsInstance(obj, ztpserver.types.String)
        self.assertEqual(repr(obj), "String(choices=one,two)")
        self.assertEqual(obj('one'), 'one')

    def test_call_string_with_choices_invalid(self):
        obj = ztpserver.types.String(['one', 'two'])
        self.assertIsInstance(obj, ztpserver.types.String)
        self.assertEqual(repr(obj), "String(choices=one,two)")
        self.assertRaises(ValueError, obj, 'three')

    def test_create_boolean_defaults(self):
        obj = ztpserver.types.Boolean()
        self.assertIsInstance(obj, ztpserver.types.Boolean)
        self.assertEqual(repr(obj), 'Boolean')

    def test_call_boolean_true_values_valid(self):
        obj = ztpserver.types.Boolean()
        self.assertIsInstance(obj, ztpserver.types.Boolean)
        values = obj.TRUEVALUES + ['YES', 'TRUE', True, 1, 'ON']
        for value in values:
            obj(value)

    def test_call_boolean_false_values_valid(self):
        obj = ztpserver.types.Boolean()
        self.assertIsInstance(obj, ztpserver.types.Boolean)
        values = obj.FALSEVALUES + ['NO', 'FALSE', False, 0, 'OFF']
        for value in values:
            obj(value)

    def test_call_boolean_invalid_values(self):
        obj = ztpserver.types.Boolean()
        self.assertIsInstance(obj, ztpserver.types.Boolean)
        values = ['test', 100, ['one', 'two']]
        for value in values:
            self.assertRaises(ValueError, obj, value)

    def test_create_integer_defaults(self):
        obj = ztpserver.types.Integer()
        self.assertIsInstance(obj, ztpserver.types.Integer)
        self.assertEqual(repr(obj), "Integer(min_value=None, max_value=None)")

    def test_create_integer_min_value(self):
        obj = ztpserver.types.Integer(min_value=1)
        self.assertEqual(obj.min_value, 1)
        self.assertIsNone(obj.max_value)

    def test_create_integer_max_value(self):
        obj = ztpserver.types.Integer(max_value=1)
        self.assertEqual(obj.max_value, 1)
        self.assertIsNone(obj.min_value)

    def test_create_integer_range(self):
        obj = ztpserver.types.Integer(min_value=1, max_value=1)
        self.assertEqual(obj.min_value, 1)
        self.assertEqual(obj.max_value, 1)

    def test_call_integer_defaults_valid(self): #pylint: disable=R0201
        obj = ztpserver.types.Integer()
        for value in [0, 1.0, 1.1, '65535', True]:
            obj(value)

    def test_call_integer_defaults_invalid(self):
        obj = ztpserver.types.Integer()
        self.assertRaises(ValueError, obj, 'string')
        self.assertRaises(TypeError, obj, [1, 2])

    def test_call_integer_min_value_valid(self):
        obj = ztpserver.types.Integer(min_value=1)
        self.assertEqual(obj(1), 1)
        self.assertEqual(obj('1'), 1)

    def test_call_integer_min_value_invalid(self):
        obj = ztpserver.types.Integer(min_value=1)
        self.assertRaises(ValueError, obj, 0)
        self.assertRaises(ValueError, obj, '0')

    def test_call_integer_max_value_valid(self):
        obj = ztpserver.types.Integer(max_value=1)
        self.assertEqual(obj(1), 1)
        self.assertEqual(obj('1'), 1)

    def test_call_integer_max_value_invalid(self):
        obj = ztpserver.types.Integer(max_value=1)
        self.assertRaises(ValueError, obj, 2)
        self.assertRaises(ValueError, obj, '2')

    def test_call_integer_range_valid(self):
        obj = ztpserver.types.Integer(min_value=1, max_value=2)
        self.assertEqual(obj(1), 1)

    def test_call_integer_range_invalid(self):
        obj = ztpserver.types.Integer(min_value=1, max_value=2)
        self.assertRaises(ValueError, obj, 3)

    def test_create_list_defaults(self):
        obj = ztpserver.types.List()
        self.assertIsInstance(obj, ztpserver.types.List)
        self.assertEqual(repr(obj), 'List(delimiter=,)')





if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
'''
    MODULE:
        ztpserver.respository

    AUTHOR:
        Arista Networks

    DESCRIPTION:
        The repository module provides read and write access to files
        for ztpserver.  The repository module can perform basic file
        system like functionality for performing basid CRUD on
        files and well as reading and writing specific file contents.

    :copyright: Copyright (c) 2015, Arista Networks
    :license: BSD, see LICENSE for more details

'''

import hashlib
import logging
import mimetypes
import os

import ztpserver.serializers

from ztpserver.serializers import SerializerError

log = logging.getLogger(__name__)   #pylint: disable=C0103



def create_repository(path):
    if not os.path.exists(path):
        raise RepositoryError('%s not found' % path)
    return Repository(path)


class RepositoryError(Exception):
    ''' Base exception class for :py:class:`Repository` '''
    pass

class FileObjectError(Exception):
    ''' Base exception class for :py:class:`FileObject` '''
    pass

class FileObjectNotFound(RepositoryError):
    ''' Raised when a requested file is not found in the repository.  This
    exception is a subclass of :py:class:`RespositoryError`
    '''
    pass



class FileObject(object):
    ''' The :py:class:`FileObject` represents a single file entity in the
    repository.   The instance provides convienent methods to read and write
    contents to the file using a specified serialization
    '''

    def __init__(self, name, path=None, **kwargs):
        ''' The initialize method for :py:class:`FileObject`

        :param name: the name of the file
        :type name: str
        :param path: the base path of the file
        :type path: str
        :param content_type: the content type of the file (optional)
        :type content_type: str
        :returns: object

        '''
        self.name = name
        if path is not None:
            self.name = os.path.join(path, name)

        self.type, self.encoding = mimetypes.guess_type(self.name)
        self.content_type = kwargs.get('content_type')

    def __repr__(self):
        return 'FileObject(name=%s, type=%s, encoding=%s, content_type=%s)' % \
               (self.name, self.type, self.encoding, self.content_type)

    def read(self, content_type=None, node_id=None):
        ''' Reads the contents from the file system

        :param content_type: defines the content_type of the file used to
                             deserialize the object
        :type content_type: str
        :returns: object
        :raises: FileObjectError

        The read method will read the file from the file system, deserializing
        the contents as specified by the content_type argument.  If the
        content_type argument is not specified, the read method will read
        the file as text. If any errors occur, a FileObjectError is raised.

        '''
        try:
            self.content_type = content_type
            return ztpserver.serializers.load(self.name, content_type,
                                              node_id)
        except SerializerError as err:
            raise FileObjectError(err.message)

    def write(self, contents, content_type=None):
        ''' Writes the contents to the file

        :param contents: specifies the contents to be written to the file
        :type contents: str
        :param content_type: defines the serialization format to use when
                             saving the file
        :type content_type: str
        :returns: None
        :raises: FileObjectError

        The write method takes the contents argument and writes it to the file
        using the serialization specified in the content_type argument.  If
        the content_type argument is not specified, the contents are written
        as string text.  This method will overwrite any contents that
        previously existed for the FileObj instance.  If any errors are
        encountered during the write operation, a FileObjectError is raised

        '''
        try:
            ztpserver.serializers.dump(contents, self.name, content_type)
            self.content_type = content_type
        except SerializerError as err:
            raise FileObjectError(err.message)

    def size(self):
        ''' Returns the size of the object in bytes.

        :raises: IOError
        '''
        return os.path.getsize(self.name)

    def hash(self):
        ''' Returns the SHA1 hash of the object.

        :raises: IOError
        '''

        sha1 = hashlib.sha1()
        sha1.update(open(self.name).read())       #pylint: disable=E1101
        return sha1.hexdigest()

class Repository(object):
    ''' The Respository class represents a repository of :py:class:`FileObject`
    instances.  It is an abstract wrapper providing the ability to interact
    with persistently stored files.
    '''

    def __init__(self, path):
        ''' The initialize method for :py:class:`Repository`

        :param path: the base path of the repository
        :type path: str
        :returns: object

        '''
        self.path = path

    def __repr__(self):
        return "Repository(path=%s)" % self.path

    def expand(self, file_path):
        ''' Expands a file_path to the full path to a file object

        :param file_path: the file path to expand
        :type file_path: str
        :returns: str -- the full path to the file

        This method is used to transform a relative file path into an
        absolute file path for identifying a file object resource

        '''
        if file_path == '/':
            file_path = self.path
        elif not str(file_path).startswith(self.path):
            file_path = file_path[1:] if file_path[0] == '/' else file_path
            file_path = os.path.join(self.path, file_path)
        return file_path

    def add_folder(self, folder_path):
        ''' Add a new folder to the repository

        :param folder_path: the full path of the folder to add
        :type folder_path: str
        :returns: str -- the full path to the new folder
        :raises: RespositoryError

        '''
        try:
            folder_path = self.expand(folder_path)
            os.makedirs(folder_path, 0774)
            return folder_path
        except OSError as err:
            log.error('Failed to add folder %s (%s)' %
                      (folder_path, err))
            raise RepositoryError('Failed to add folder %s (%s)' %
                                  (folder_path, err))

    def add_file(self, file_path, contents=None, content_type=None):
        ''' Adds a new :py:class:`FileObject` to the repository

        :param file_path: the full path of the file to add
        :type file_path: str
        :param contents: the contents to write to the file
        :type contents: str
        :param content_type: specifies the serialization to use for the file
        :type content_type: str
        :returns: :py:class:`FileObject`
        :raises: RespositoryError

        The add_file method allows for a new file to be added to the
        respository.  If the file already exists, it is returned as an instance
        of :py:class:`FileObject`.   If the file doesn't already exist and
        the contents argument is not None, then the file is created and
        the contents written to the file.  The content_type argument provides
        the serialization to be used when saving the file.

        '''
        file_path = self.expand(file_path)
        obj = FileObject(file_path)
        if contents:
            obj.write(contents, content_type)
        return obj

    def exists(self, file_path):
        ''' Returns boolean if the file_path exists in the repository

        :param file_path: the file_path to check for existence
        :type file_path: str
        :returns: boolean -- True if it exists otherwise False

        '''
        file_path = self.expand(file_path)
        return os.path.exists(file_path)

    def get_file(self, file_path):
        ''' Returns an intance of :py:class:`FileObject` if it exists

        :param file_path: the file path of the instance to return
        :type file_path: str
        :returns: instance of :py:class:`FileObject`
        :raises: FileObjectNotFound

        This method will retrieve a file object instance if it exists in the
        repository.  If the file does not exist then an error is raised

        '''
        file_path = self.expand(file_path)
        if not self.exists(file_path):
            raise FileObjectNotFound('file not found (%s)' % file_path)
        return FileObject(file_path)

    def delete_file(self, file_path):
        ''' Deletes an existing file in the respository

        :param file_path: the file path of the instance to delete
        :type file_path: str
        :returns: None
        :raises: RepositoryError

        '''
        try:
            file_path = self.expand(file_path)
            os.remove(file_path)
        except (OSError, IOError) as err:
            log.error('Failed to delete file %s (%s)' %
                      (file_path, err))
            raise RepositoryError('Failed to delete file %s (%s)' %
                                  (file_path, err))



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
# pylint: disable=C0103
#

import argparse
import logging
import os
import re
import sys

from wsgiref.simple_server import make_server


from ztpserver import config, controller

from ztpserver.serializers import load, dump
from ztpserver.validators import NeighbordbValidator
from ztpserver.constants import CONTENT_TYPE_YAML
from ztpserver.topology import FUNC_RE, neighbordb_path
from ztpserver.utils import all_files
from ztpserver.resources import resource_plugins

log = logging.getLogger('ztpserver')
log.setLevel(logging.DEBUG)
log.addHandler(logging.NullHandler())

def enable_handler_console(level=None):
    ''' Enables logging to stdout '''
    
    logging_fmt = config.runtime.default.console_logging_format
    formatter = logging.Formatter(logging_fmt)

    ch = logging.StreamHandler()
    ch.tag = 'console'

    
    for handler in log.handlers:
        if 'tag' in handler.__dict__ and handler.tag == ch.tag:
            # Handler previously added
            return

    level = level or 'DEBUG'
    level = str(level).upper()
    level = logging.getLevelName(level)
    ch.setLevel(level)
    ch.setFormatter(formatter)

    log.addHandler(ch)

def python_supported():
    ''' Returns True if the current version of the python runtime is valid '''
    return sys.version_info > (2, 7) and sys.version_info < (3, 0)

logging_started = False
def start_logging(debug):
    ''' reads the runtime config and starts logging if enabled '''
    global logging_started     #pylint: disable=W0603
    if logging_started:
        return

    if config.runtime.default.logging:
        if config.runtime.default.console_logging:
            enable_handler_console('DEBUG' if debug else 'INFO')

    logging_started = True

def load_config(conf=None):
    conf = conf or config.GLOBAL_CONF_FILE_PATH
    conf = os.environ.get('ZTPS_CONFIG', conf)

    if os.path.exists(conf):
        log.info('Loading config file: %s' % conf)
        config.runtime.read(conf)

def start_wsgiapp(config_file=None, debug=False):
    ''' Provides the entry point into the application for wsgi compliant
    servers.   Accepts an optional argument ``config_file``.   The
    ``config_file`` keyword argument specifies the path the server
    configuration file.  The default value is /etc/ztpserver/ztpserver.conf.

    :param config_file: string path pointing to configuration file
    :param debug: boolean set debug level logging? (Default: False)
    :return: a wsgi application object

    '''
    load_config(config_file)
    start_logging(debug)

    try:
        version = open(config.VERSION_FILE_PATH).read().split()[0].strip()
    except Exception:  # pylint: disable=W0703
        version = 'N/A'

    log.info('Starting ZTPServer v%s...' % version)

    log.info('Logging started for ztpserver')
    log.info('Using repository %s', config.runtime.default.data_root)

    if not python_supported():
        raise SystemExit('ERROR: ZTPServer requires Python 2.7')

    return controller.Router()

def run_server(version, config_file, debug):
    ''' The :py:func:`run_server` is called by the main command line routine to
    run the server as standalone.   This function accepts a single argument
    that points towards the configuration file describing this server

    This function will block on the active thread until stopped.

    :param conf: string path pointing to configuration file
    '''
    app = start_wsgiapp(config_file, debug)

    host = config.runtime.server.interface
    port = config.runtime.server.port

    log.info('URL: http://%s:%s' % (host, port))

    httpd = make_server(host, port, app)

    log.info('Starting ZTPServer v%s on http://%s:%s' % 
             (version, host, port))

    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        log.info('Shutdown...')

def validate_neighbordb():
    # Validating neighbordb
    validator = NeighbordbValidator('N/A')
    neighbordb = neighbordb_path()
    print 'Validating neighbordb (\'%s\')...' % neighbordb
    try:
        validator.validate(load(neighbordb, CONTENT_TYPE_YAML,
                                'validator'))
        total_patterns = len(validator.valid_patterns) + \
            len(validator.invalid_patterns)
            
        if validator.invalid_patterns:
            print '\nERROR: Failed to validate neighbordb patterns'
            print '   Invalid Patterns (count: %d/%d)' % \
                (len(validator.invalid_patterns),
                 total_patterns)
            print '   ---------------------------'
            for index, pattern in enumerate(
                sorted(validator.invalid_patterns)):
                print '   [%d] %s' % (index, pattern[1])
        else:
            print 'Ok!'            
    except Exception as exc:        #pylint: disable=W0703
        print 'ERROR: Failed to validate neighbordb\n%s' % exc

def validate_definitions():
    data_root = config.runtime.default.data_root

    print '\nValidating definitions...'

    for definition in all_files(os.path.join(data_root, 
                                             'definitions')):
        print 'Validating %s...' % definition,
        try:
            def_data = load(definition, CONTENT_TYPE_YAML,
                            'validator')

            resources = re.findall(FUNC_RE, 
                                   str(def_data))

            # Validating plugins
            plugins = resource_plugins()
            missing_plugins = set([x for (x, _) in resources 
                                   if x not in plugins])
            if missing_plugins:
                plugins_path = os.path.join(data_root, 
                                            'plugins')
                print ''
                for plugin in missing_plugins:
                    print 'ERROR: Plugin \'%s\' configured in \'%s\' ' \
                        'is missing from \'%s\'!' % \
                        (plugin, definition, plugins_path)

            # Special validation for 'allocate' plugin
            resources_path = os.path.join(data_root, 
                                          'resources')
            resource_files = [x.split('/')[-1] 
                              for x in os.listdir(resources_path)]
            missing_resources = [x for (y, x) in resources 
                                 if x not in resource_files and
                                 y == 'allocate']
            if missing_resources:
                if not missing_plugins:
                    print ''
                for res in missing_resources:
                    print 'ERROR: Resource file \'%s\' configured in \'%s\' ' \
                        'is missing from \'%s\'!' % \
                        (res, definition, resources_path)
            else:
                print 'Ok!'
        except Exception as exc:        #pylint: disable=W0703            
            print '\nERROR: Failed to validate %s\n%s' % \
                (definition, exc)

def validate_resources(raiseException=False):
    data_root = config.runtime.default.data_root

    print '\nValidating resources...'
    for resource in all_files(os.path.join(data_root, 
                                           'resources')):
        print 'Validating %s...' % resource,
        try:
            load(resource, CONTENT_TYPE_YAML,
                 'validator')
            print 'Ok!'
        except Exception as exc:        #pylint: disable=W0703 
            print '\nERROR: Failed to validate %s\n%s' % \
                (resource, exc)
            if raiseException:
                raise exc

def validate_nodes():
    data_root = config.runtime.default.data_root

    print '\nValidating nodes...'
    for filename in [x for x in all_files(os.path.join(data_root, 
                                                       'nodes'))
                     if x.split('/')[-1] in ['definition',
                                             'pattern']]:
        print 'Validating %s...' % filename,
        try:
            load(filename, CONTENT_TYPE_YAML,
                 'validator')
            print 'Ok!'
        except Exception as exc:        #pylint: disable=W0703            
            print '\nERROR: Failed to validate %s\n%s' % \
                (filename, exc)

def clear_resources(debug):
    start_logging(debug)

    try:
        validate_resources(raiseException=True)
    except Exception:                   #pylint: disable=W0703 
        sys.exit('ERROR: Unable to clear resources because of validation error')

    data_root = config.runtime.default.data_root

    print '\nClearing resources...'
    for resource in all_files(os.path.join(data_root, 
                                           'resources')):
        print 'Clearing %s...' % resource,
        try:
            contents = load(resource, CONTENT_TYPE_YAML,
                            'clear_resource')
            for key in contents:
                contents[key] = 'None'
            dump(contents, resource, CONTENT_TYPE_YAML,
                 'clear_resource')
            print 'Ok!'            
        except Exception as exc:        #pylint: disable=W0703            
            print '\nERROR: Failed to clear %s\n%s' % \
                (resource, exc)
    
def run_validator(debug):
    start_logging(debug)

    validate_neighbordb()
    validate_definitions()
    validate_resources()
    validate_nodes()

def main():
    ''' The :py:func:`main` is the main entry point for the ztpserver if called
    from the commmand line.   When called from the command line, the server is
    running in standalone mode as opposed to using the :py:func:`application` to
    run under a python wsgi compliant server
    '''

    usage = 'ztpserver [options]'

    parser = argparse.ArgumentParser(usage=usage)

    parser.add_argument('--version', '-v',
                        action='store_true',
                        help='Displays the version information')

    parser.add_argument('--conf', '-c',
                        type=str,
                        default=config.GLOBAL_CONF_FILE_PATH,
                        help='Specifies the configuration file to use')

    parser.add_argument('--validate-config', '-V',
                        action='store_true',
                        help='Validates config files')

    parser.add_argument('--debug',
                        action='store_true',
                        help='Enables debug output to the STDOUT')

    parser.add_argument('--clear-resources', '-r',
                        action='store_true',
                        help='Clears all resource files')


    args = parser.parse_args()

    version = 'N/A'
    try:
        version = open(config.VERSION_FILE_PATH).read().split()[0].strip()
    except IOError:
        pass

    if args.version:
        print 'ZTPServer version %s' % version

    if args.validate_config:
        run_validator(args.debug)

    if args.clear_resources:
        clear_resources(args.debug)

    if args.version or args.validate_config or args.clear_resources:
        sys.exit()

    return run_server(version, args.conf, args.debug)



#!/usr/bin/env python
#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#  - Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#  - Neither the name of Arista Networks nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#pylint: disable=R0904,F0401

import os
import os.path
import unittest

from client_test_lib import Bootstrap
from client_test_lib import file_log, remove_file
from client_test_lib import startup_config_action
from client_test_lib import fail_action, print_action, random_string
from client_test_lib import erroneous_action, missing_main_action
from client_test_lib import wrong_signature_action, exception_action
from client_test_lib import raise_exception

class ServerNotRunningTest(unittest.TestCase):

    def test(self):
        bootstrap = Bootstrap(server='unkown')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.server_connection_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()


class ConfigRequestErrorTest(unittest.TestCase):

    def test_status(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response(status=201)
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_content_type(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response(content_type='text/plain')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_status_content_type(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response(
            status=201,
            content_type='text/plain')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()


class EAPIErrorTest(unittest.TestCase):

    def test(self):
        bootstrap = Bootstrap(eapi_port=54321)
        bootstrap.ztps.set_config_response()
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_configured())
            self.failUnless(bootstrap.eapi_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()


class CheckNodeErrorTest(unittest.TestCase):

    def test_status(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response(status=200)
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_content_type(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response(
            content_type='text/plain')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_status_content_type(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response(
            status=200,
            content_type='text/plain')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_node_not_found(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response(status=400)
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.node_not_found_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_bogus_location(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response(location='bogus_location')
        bootstrap.ztps.set_definition_response()
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.server_connection_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()


class DefinitionErrorTest(unittest.TestCase):

    def test_definition_missing(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.server_connection_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_status(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(status=201)

        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_content_type(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            content_type='text/plain')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_status_content_type(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            status=201,
            content_type='text/plain')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_bogus_definition_response(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_bogus_definition_response()
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.invalid_definition_format())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_topology_check(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            status=400,
            content_type='text/html')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.toplogy_check_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()


class MissingStartupConfigTest(unittest.TestCase):

    def test(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response()
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.missing_startup_config_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()


class FactoryDefaultTest(unittest.TestCase):

    def test(self):
        bootstrap = Bootstrap()

        open(bootstrap.rc_eos, 'w').write(random_string())
        open(bootstrap.boot_extensions, 'w').write(random_string())
        os.makedirs(bootstrap.boot_extensions_folder)
        open('%s/%s' % (bootstrap.boot_extensions_folder, 
                        random_string()),
             'w').write(random_string())

        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response()
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.missing_startup_config_failure())
            self.failIf(os.path.exists(bootstrap.rc_eos))
            self.failIf(os.path.exists(bootstrap.boot_extensions))
            self.failIf(os.path.exists(bootstrap.boot_extensions_folder))

            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()


class FileLogConfigTest(unittest.TestCase):

    def test(self):
        filenames = {
            'DEBUG' : '/tmp/ztps-log-%s-debug' % os.getpid(),
            'ERROR' : '/tmp/ztps-log-%s-error' % os.getpid(),
            'INFO' : '/tmp/ztps-log-%s-info' % os.getpid(),
            'bogus' : '/tmp/ztps-log-%s-bogus' % os.getpid()
            }

        logging = []
        for level, filename in filenames.iteritems():
            logging += {'destination' : 'file:%s' % filename,
                        'level' : level},

        for filename in filenames.itervalues():
            self.failIf(os.path.isfile(filename))

        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response(logging=logging)
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response()
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.missing_startup_config_failure())
            for filename in filenames.itervalues():
                self.failUnless(file_log(filename))
            self.assertEquals(file_log(filenames['DEBUG'],
                                       ignore_string='SyslogManager'),
                              file_log(filenames['bogus'],
                                       ignore_string='SyslogManager'))
            self.assertEquals(file_log(filenames['DEBUG'],
                                       ignore_string='SyslogManager'),
                              file_log(filenames['INFO'],
                                       ignore_string='SyslogManager'))
            self.failIfEqual(file_log(filenames['DEBUG'],
                                       ignore_string='SyslogManager'),
                             file_log(filenames['ERROR'],
                                       ignore_string='SyslogManager'))
            self.failUnless(set(file_log(filenames['ERROR'])).issubset(
                    set(file_log(filenames['DEBUG']))))
            for filename in filenames.itervalues():
                remove_file(filename)
                self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()



class ActionFailureTest(unittest.TestCase):

    def action_fail_test(self, action):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'test_action'}])
        bootstrap.ztps.set_action_response('test_action', action)
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.action_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_return_code_action_failed(self):
        self.action_fail_test(fail_action())

    def test_erroneous_action_failed(self):
        self.action_fail_test(erroneous_action())

    def test_missing_main_action_failed(self):
        self.action_fail_test(missing_main_action())

    def test_wrong_sig_action_failed(self):
        self.action_fail_test(wrong_signature_action())

    def test_exception_action_failed(self):
        self.action_fail_test(exception_action())

    def test_status(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'test_action'}])
        bootstrap.ztps.set_action_response('test_action', print_action(),
                                           status=201)
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_content_type(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'test_action'}])
        bootstrap.ztps.set_action_response('test_action', print_action(),
                                           content_type='test/plain')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_status_content_type(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'test_action'}])
        bootstrap.ztps.set_action_response('test_action', print_action(),
                                           status=201,
                                           content_type='test/plain')
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.unexpected_response_failure())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_action_failure_log(self):
        log = '/tmp/ztps-log-%s-debug' % os.getpid()

        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response(logging=[
                {'destination' : 'file:%s' % log,
                 'level' : 'DEBUG'},])
        bootstrap.ztps.set_node_check_response()

        text_onstart = random_string()
        text_onsuccess = random_string()
        text_onfailure = random_string()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'test_action',
                      'onstart' : text_onstart,
                      'onsuccess' : text_onsuccess,
                      'onfailure' : text_onfailure},
                     ])
        bootstrap.ztps.set_action_response('test_action',
                                           fail_action())
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.action_failure())
            self.failIf(bootstrap.error)
            log = ''.join(file_log(log))
            self.failUnless('test_action: %s' % text_onstart in log)
            self.failUnless('test_action: %s' % text_onsuccess not in log)
            self.failUnless('test_action: %s' % text_onfailure in log)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()


class BootstrapSuccessTest(unittest.TestCase):

    def test_success(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'}])
        bootstrap.ztps.set_action_response('startup_config_action',
                                           startup_config_action())
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.success())
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_multiple_actions(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'},
                     {'action' : 'print_action_1'},
                     {'action' : 'print_action_2'}])
        bootstrap.ztps.set_action_response('startup_config_action',
                                           startup_config_action())

        text_1 = random_string()
        bootstrap.ztps.set_action_response('print_action_1',
                                           print_action(text_1))
        text_2 = random_string()
        bootstrap.ztps.set_action_response('print_action_2',
                                           print_action(text_2))
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.success())
            self.failUnless(text_1 in bootstrap.output)
            self.failUnless(text_2 in bootstrap.output)
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_duplicate_actions(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'},
                     {'action' : 'print_action'},
                     {'action' : 'print_action'}])
        bootstrap.ztps.set_action_response('startup_config_action',
                                           startup_config_action())

        text = random_string()
        bootstrap.ztps.set_action_response('print_action',
                                           print_action(text))
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.success())
            self.failUnless(bootstrap.output.count('Downloading action '
                                                   'print_action') == 1)
            self.failUnless(bootstrap.output.count('Executing action '
                                                   'print_action') == 2)
            self.failUnless(bootstrap.output.count(text) == 2)
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_attribute_copy(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        text = random_string()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'},
                     {'action' : 'print_action',
                      'attributes' : {'print_action-attr' : text}}])
        bootstrap.ztps.set_action_response('startup_config_action',
                                           startup_config_action())
        bootstrap.ztps.set_action_response('print_action',
                                           print_action(use_attribute=True,
                                                        create_copy=True))
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.success())
            self.failUnless(text in bootstrap.output)
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_attribute(self):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response()
        bootstrap.ztps.set_node_check_response()
        text = random_string()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'},
                     {'action' : 'print_action',
                      'attributes' : {'print_action-attr':text}}])
        bootstrap.ztps.set_action_response('startup_config_action',
                                           startup_config_action())
        bootstrap.ztps.set_action_response('print_action',
                                           print_action(use_attribute=True))
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.success())
            self.failUnless(text in bootstrap.output)
            self.failIf(bootstrap.error)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_action_success_log(self):
        log = '/tmp/ztps-log-%s-debug' % os.getpid()

        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response(logging=[
                {'destination' : 'file:%s' % log,
                 'level' : 'DEBUG'},])
        bootstrap.ztps.set_node_check_response()

        text_onstart = random_string()
        text_onsuccess = random_string()
        text_onfailure = random_string()
        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action',
                      'onstart' : text_onstart,
                      'onsuccess' : text_onsuccess,
                      'onfailure' : text_onfailure,
                      }])
        bootstrap.ztps.set_action_response('startup_config_action',
                                           startup_config_action())
        bootstrap.start_test()

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.success())
            self.failIf(bootstrap.error)
            log = ''.join(file_log(log))
            self.failUnless('startup_config_action: %s' % text_onstart in log)
            self.failUnless('startup_config_action: %s' % text_onsuccess in log)
            self.failUnless('startup_config_action: %s' % 
                            text_onfailure not in log)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()


if __name__ == '__main__':
    unittest.main()



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


class String(object):

    def __init__(self, choices=None):
        if choices:
            choices = [str(c) for c in choices]
        self.choices = choices

    def __call__(self, value):
        value = str(value)

        if self.choices and value not in self.choices:
            raise ValueError('Invalid string value: %s is not one of %s' %
                             (value, self.choices))

        return value

    def __repr__(self):
        obj = 'String'
        if self.choices:
            obj += '(choices=%s)' % ','.join(self.choices)
        return obj


class Boolean(object):

    TRUEVALUES = ['yes', 'true', '1', 'on']
    FALSEVALUES = ['no', 'false', '0', 'off']

    def __call__(self, value):
        if str(value).lower() in self.TRUEVALUES:
            return True
        elif str(value).lower() in self.FALSEVALUES:
            return False
        else:
            raise ValueError('Invalid boolean value: %s' % value)

    def __repr__(self):
        return 'Boolean'


class Integer(object):

    def __init__(self, min_value=None, max_value=None):
        self.min_value = min_value
        self.max_value = max_value

    def __call__(self, value):
        try:
            value = int(value)
        except ValueError:
            raise ValueError('Invalid integer value: %s' % value)

        if self.min_value is not None and value < self.min_value:
            raise ValueError('Invalid integer value: %s (min is %s)' %
                             (value, self.min_value))

        if self.max_value is not None and value > self.max_value:
            raise ValueError('Invalid integer value: %s (max is %s)' %
                             (value, self.max_value))

        return value

    def __repr__(self):
        return 'Integer(min_value=%s, max_value=%s)' % \
               (self.min_value, self.max_value)


class List(object):

    def __init__(self, delimiter=','):
        self.delimiter = delimiter

    def __call__(self, value):
        if isinstance(value, list):
            return value

        return str(value).split(self.delimiter)

    def __repr__(self):
        return 'List(delimiter=%s)' % self.delimiter



#!/usr/bin/env python
#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#  - Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#  - Neither the name of Arista Networks nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#pylint: disable=R0904,F0401,W0232,E1101

import sys
import unittest

sys.path.append('test/client')

from client_test_lib import Bootstrap, ActionFailureTest
from client_test_lib import get_action, random_string
from client_test_lib import startup_config_action
from client_test_lib import raise_exception

class FailureTest(ActionFailureTest):

    def test_missing_smarthost(self):
        self.basic_test('send_email',
                        'Missing attribute(\'smarthost\')')

    def test_missing_sender(self):
        self.basic_test('send_email',
                        'Missing attribute(\'sender\')',
                        attributes={'smarthost' :
                                    random_string()})
    def test_missing_receivers(self):
        self.basic_test('send_email',
                        'Missing attribute(\'receivers\')',
                        attributes={'smarthost' :
                                    random_string(),
                                    'sender' :
                                    random_string()})


class SuccessTest(unittest.TestCase):

    def test_success(self):
        bootstrap = Bootstrap(ztps_default_config=True)

        smarthost = "%s:2525" % str(bootstrap.server).split(':')[0]

        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'},
                     {'action' : 'test_action',
                      'attributes' : {
                        'smarthost': smarthost,
                        'sender': 'ztps@localhost',
                        'receivers': ['ztps@localhost']}}])


        bootstrap.ztps.set_action_response('startup_config_action',
                                           startup_config_action())

        action = get_action('send_email')
        bootstrap.ztps.set_action_response('test_action',
                                           action)

        bootstrap.start_test()
        try:
            self.failUnless(bootstrap.success())
        except Exception as exception: #pylint: disable=W0703
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(exception)
        finally:
            bootstrap.end_test()



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#  - Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#  - Neither the name of Arista Networks nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#pylint: disable=R0904,F0401

import unittest

from client_test_lib import Bootstrap
from client_test_lib import raise_exception


class XmppConfigTest(unittest.TestCase):

    @classmethod
    def bootstrap(cls, xmpp):
        bootstrap = Bootstrap()
        bootstrap.ztps.set_config_response(xmpp=xmpp)
        bootstrap.ztps.set_node_check_response()
        bootstrap.ztps.set_definition_response()
        bootstrap.start_test()
        return bootstrap

    def xmpp_sanity_test(self, xmpp):
        bootstrap = self.bootstrap(xmpp)

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.missing_startup_config_failure())
            self.failIf(bootstrap.error)
            self.failIf('XmppClient' not in bootstrap.output)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()

    def test_full(self):
        self.xmpp_sanity_test({'server' : 'test-server',
                               'port' : 112233,
                               'username' : 'test-username',
                               'password' : 'test-password',
                               'domain' :   'test-domain',
                               'rooms' : ['test-room-1', 'test-room-2']})

    def test_msg_type_debug(self):
        self.xmpp_sanity_test({'server' : 'test-server',
                               'port' : 112233,
                               'username' : 'test-username',
                               'password' : 'test-password',
                               'domain' :   'test-domain',
                               'rooms' : ['test-room-1', 'test-room-2'],
                               'msg_type' : 'debug'})

    def test_msg_type_info(self):
        self.xmpp_sanity_test({'server' : 'test-server',
                               'port' : 112233,
                               'username' : 'test-username',
                               'password' : 'test-password',
                               'domain' :   'test-domain',
                               'rooms' : ['test-room-1', 'test-room-2'],
                               'msg_type' : 'debug'})

    def test_partial(self):
        self.xmpp_sanity_test({'rooms' : ['test-room-1'],
                               'username' : 'test-username',
                               'password' : 'test-password',
                               'domain' :   'test-domain'})

    def test_erroneous_msg_type(self):
        bootstrap = self.bootstrap({'server' : 'test-server',
                                    'port' : 112233,
                                    'username' : 'test-username',
                                    'password' : 'test-password',
                                    'domain' :   'test-domain',
                                    'rooms' : ['test-room-1', 'test-room-2'],
                                    'msg_type' : 'bogus'})

        try:
            self.failUnless(bootstrap.eapi_node_information_collected())
            self.failUnless(bootstrap.missing_startup_config_failure())
            self.failIf(bootstrap.error)
            self.failIf('XMPP configuration failed because of '
                        'unexpected \'msg_type\''
                        not in bootstrap.output)
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            bootstrap.end_test()



if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import os
import random
import unittest

import ztpserver.serializers as serializers

from ztpserver.constants import CONTENT_TYPE_JSON

TMP_FILE = '/tmp/test_serializers-%s' % os.getpid()

def get_data():
    result = {}
    for _ in range(random.randint(10, 100)):
        key = 'x' * random.randint(0, 10)
        value = 'x' * random.randint(0, 50)
        result[key] = value

    return result

class SerializersUnitTest(unittest.TestCase):

    def test_dump_success(self):
        pass

    def test_dumps_success(self):
        pass

    def test_load_success(self):
        pass

    def test_loads_success(self):
        pass

    @classmethod
    def test_stress(cls):
        # stress test writing and loading the same file over and
        # over again and make sure the file does not get corrupted
        # in the process
        index = 0
        while index < 100:
            index += 1
            data = get_data()
            serializers.dump(data, TMP_FILE, 
                             CONTENT_TYPE_JSON)
            assert serializers.load(TMP_FILE, 
                                    CONTENT_TYPE_JSON) == data

if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# pylint: disable=W0142

import os
import unittest
import traceback
import yaml

from ztpserver.app import enable_handler_console
from ztpserver.topology import load_neighbordb, load_file
from ztpserver.topology import Node
from ztpserver.validators import NeighbordbValidator
from ztpserver.constants import CONTENT_TYPE_YAML
from server_test_lib import enable_logging, log, random_string

TEST_DIR = 'test/neighbordb'
ID = random_string()

def debug(exc):
    # Uncomment line for debugging
    # import pdb; pdb.set_trace()
    
    raise exc

def load_node(node, content_type=CONTENT_TYPE_YAML):
    try:
        if not hasattr(node, 'items'):
            node = load_file(node, content_type, ID)
        if 'systemmac' in node:
            for symbol in [':', '.']:
                node['systemmac'] = str(node['systemmac']).replace(symbol, '')
        return Node(**node)
    except TypeError:
        log.error('Failed to load node')
    except KeyError as err:
        log.error('Failed to load node - missing attribute: %s' % err)

class NeighbordbTest(unittest.TestCase):
    # pylint: disable=C0103

    def __init__(self, test_name, ndb, **kwargs):
        self.ndb = ndb
        self.node = kwargs.get('node')
        self.tag = kwargs.get('tag')
        self.match = kwargs.get('match')
        self.filename = kwargs.get('filename')

        self.valid_patterns = kwargs.get('valid_patterns', dict())
        if not self.valid_patterns.get('nodes'):
            self.valid_patterns['nodes'] = list()
        if not self.valid_patterns.get('globals'):
            self.valid_patterns['globals'] = list()

        self.invalid_patterns = kwargs.get('invalid_patterns')

        self.longMessage = True   
        self.maxDiff = None

        super(NeighbordbTest, self).__init__(test_name)

    def _load_neighbordb(self):
        return load_neighbordb(ID, contents=self.ndb)

    def _validate_neighbordb(self):
        validator = NeighbordbValidator(ID)
        validator.validate(self.ndb)
        return (validator.valid_patterns, validator.invalid_patterns)

    def neighbordb_pattern(self):
        try:
            log.info('START: neighbordb_patterns')
            tag = 'tag=%s, fn=%s' % (self.tag, self.filename)

            (valid, failed) = self._validate_neighbordb()

            if self.invalid_patterns:
                failed_names = sorted([p[1] for p in failed])
                self.assertEqual(failed_names, 
                                 sorted(self.invalid_patterns),
                                 tag)
                self.assertEqual(len(failed), len(self.invalid_patterns), tag)

            p_all = self.valid_patterns.get('nodes') + \
                    self.valid_patterns.get('globals')
            if p_all:
                valid_actual = sorted([str(p[1]) for p in valid])
                valid_configured = sorted([str(x) for x in p_all])

                self.assertEqual(valid_actual, valid_configured, tag)

            log.info('END: neighbordb_pattern')
        except AssertionError as exc:
            print exc
            print traceback.format_exc()
            debug(exc)

    def neighbordb_topology(self):
        try:
            log.info('START: neighbordb_test')
            tag = 'tag=%s, fn=%s' % (self.tag, self.filename)

            topo = self._load_neighbordb()
            self.assertIsNotNone(topo, tag)

            p_nodes = [p.name for p in topo.get_patterns() 
                       if topo.is_node_pattern(p)]
            p_globals = [p.name for p in topo.get_patterns()
                         if topo.is_global_pattern(p)]

            self.assertEqual(sorted(self.valid_patterns['nodes']),
                             sorted(p_nodes),
                             tag)
            self.assertEqual(sorted(self.valid_patterns['globals']),
                             sorted(p_globals),
                             tag)

            log.info('END: neighbordb_test')
            return topo
        except AssertionError as exc:
            print exc
            print traceback.format_exc()
            debug(exc)

    def node_pass(self):
        try:
            log.info('START: node_pass')
            tag = 'tag=%s, fn=%s' % (self.tag, self.filename)
            
            node = load_node(self.node)
            self.assertIsNotNone(node, tag)
            
            neighbordb = self._load_neighbordb()
            self.assertIsNotNone(neighbordb, tag)
            result = neighbordb.match_node(node)
            
            self.assertTrue(result, tag)
            self.assertEqual(result[0].name, self.match, tag)
            log.info('END: node_pass')
        except AssertionError as exc:
            print exc
            print traceback.format_exc()
            debug(exc)

    def node_fail(self):
        try:
            log.info('START: node_fail')
            tag = 'tag=%s, fn=%s' % (self.tag, self.filename)

            node = load_node(self.node)
            self.assertIsNotNone(node, tag)

            neighbordb = self._load_neighbordb()
            self.assertIsNotNone(neighbordb, tag)

            result = neighbordb.match_node(node)

            self.assertFalse(result, tag)
            log.info('END: node_fail')
        except AssertionError as exc:
            print exc
            print traceback.format_exc()
            debug(exc)


def get_test_list(filepath):
    test_list = os.environ.get('TESTS', None)
    if not test_list:
        log.debug('Checking directory %s for tests', TEST_DIR)
        test_list = [os.path.join(filepath, f) for f in os.listdir(filepath)
                     if f.endswith('yml')]
    else:
        test_list = [os.path.join(filepath, f) for f in test_list.split(',')]
    return test_list

def load_tests(loader, tests, pattern):      # pylint: disable=W0613
    try:
        log.info('Start ndb tests')

        suite = unittest.TestSuite()
        test_list = get_test_list(TEST_DIR)
        log.info('Test list is %s', test_list)

        for test in test_list:
            log.info('Starting test harness %s', test)

            harness = yaml.load(open(test))
            if harness.get('debug'):
                enable_handler_console()

            tests = harness.get('tests', ['topology'])

            name = harness.get('ndb', 'neighbordb')
            filename = name.split('/')[-1]
            if filename in harness:
                ndb = harness.get(filename)
            else:
                assert os.path.exists(name)
                ndb = yaml.load(open(name))

            kwargs = dict()
            kwargs['valid_patterns'] = harness.get('valid_patterns', dict())
            kwargs['invalid_patterns'] = harness.get('invalid_patterns')
            kwargs['tag'] = harness.get('tag', filename)
            kwargs['filename'] = test

            for test in tests:
                assert test in ['pattern', 'topology']
                suite.addTest(NeighbordbTest('neighbordb_%s' % test, 
                                             ndb, **kwargs))

            if 'nodes' in harness and 'topology' in tests:
                for key in harness['nodes'].keys():
                    assert key in ['pass', 'fail']
                    entries = harness['nodes'][key]
                    if not entries is None:
                        for entry in entries:
                            filename = entry['name'].split('/')[-1]
                            kwargs['node'] = harness.get(filename, 
                                                         entry['name'])
                            kwargs['match'] = entry.get('match')
                            kwargs['tag'] = '%s:%s' % (harness.get('tag'), 
                                                       filename)
                            log.info('Adding node %s', name)
                            suite.addTest(NeighbordbTest('node_%s' % key, 
                                                         ndb, **kwargs))

    except Exception as exc:      # pylint: disable=W0703
        log.exception('Unexpected error trying to execute load_tests: %s' %
                      exc)
    else:
        return suite

if __name__ == '__main__':
    enable_logging()
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import collections
import os
import tempfile
import unittest

import ztpserver.config

class TestAttributes(unittest.TestCase):

    def test_attr_valid(self):
        obj = ztpserver.config.Attr('test')
        self.assertIsInstance(obj, ztpserver.config.Attr)

    def test_attr_invalid(self):
        self.assertRaises(TypeError, ztpserver.config.Attr)

    def test_strattr_defaults(self):
        obj = ztpserver.config.StrAttr('test')
        self.assertIsInstance(obj, ztpserver.config.StrAttr)

    def test_strattr_with_choices(self):
        obj = ztpserver.config.StrAttr('test', choices=['one', 'two'])
        self.assertIsInstance(obj, ztpserver.config.StrAttr)
        self.assertEqual(repr(obj.type), 'String(choices=one,two)')

    def test_strattr_with_default(self):
        obj = ztpserver.config.StrAttr('test', default='test')
        self.assertIsInstance(obj, ztpserver.config.StrAttr)
        self.assertEqual(obj.default, 'test')

    def test_strattr_with_group(self):
        obj = ztpserver.config.StrAttr('test', group='test')
        self.assertIsInstance(obj, ztpserver.config.StrAttr)
        self.assertEqual(obj.group, 'test')

    def test_strattr_with_choices_default_valid(self):
        obj = ztpserver.config.StrAttr('test', choices=['one', 'two'],
            default='one')
        self.assertIsInstance(obj, ztpserver.config.StrAttr)
        self.assertEqual(repr(obj.type), 'String(choices=one,two)')
        self.assertEqual(obj.default, 'one')

    def test_strattr_with_choices_default_invalid(self):
        self.assertRaises(ValueError, ztpserver.config.StrAttr,
            'test', choices=['one', 'two'], default='three')

    def test_strattr_with_choices_default_group(self):
        obj = ztpserver.config.StrAttr('test', choices=['one', 'two'],
            group='test', default='one')
        self.assertIsInstance(obj, ztpserver.config.StrAttr)
        self.assertEqual(repr(obj.type), 'String(choices=one,two)')
        self.assertEqual(obj.default, 'one')
        self.assertEqual(obj.group, 'test')

    def test_intattr_defaults(self):
        obj = ztpserver.config.IntAttr('test')
        self.assertIsInstance(obj, ztpserver.config.IntAttr)

    def test_intattr_with_min_value(self):
        obj = ztpserver.config.IntAttr('test', min_value=1)
        self.assertIsInstance(obj, ztpserver.config.IntAttr)
        self.assertEqual(repr(obj.type), "Integer(min_value=1, max_value=None)")

    def test_intattr_with_max_value(self):
        obj = ztpserver.config.IntAttr('test', max_value=1)
        self.assertIsInstance(obj, ztpserver.config.IntAttr)
        self.assertEqual(repr(obj.type), "Integer(min_value=None, max_value=1)")

    def test_intattr_with_min_and_max_value(self):
        obj = ztpserver.config.IntAttr('test', min_value=1, max_value=1)
        self.assertIsInstance(obj, ztpserver.config.IntAttr)
        self.assertEqual(repr(obj.type), "Integer(min_value=1, max_value=1)")

    def test_intattr_with_default(self):
        obj = ztpserver.config.IntAttr('test', default=1)
        self.assertIsInstance(obj, ztpserver.config.IntAttr)
        self.assertEqual(obj.default, 1)

    def test_intattr_with_min_valid_default(self):
        obj = ztpserver.config.IntAttr('test', min_value=1, default=1)
        self.assertIsInstance(obj, ztpserver.config.IntAttr)
        self.assertEqual(obj.default, 1)

    def test_intattr_with_max_valid_default(self):
        obj = ztpserver.config.IntAttr('test', max_value=1, default=1)
        self.assertIsInstance(obj, ztpserver.config.IntAttr)
        self.assertEqual(obj.default, 1)

    def test_intattr_with_range_valid_default(self):
        obj = ztpserver.config.IntAttr('test', min_value=1, max_value=1,
            default=1)
        self.assertIsInstance(obj, ztpserver.config.IntAttr)
        self.assertEqual(obj.default, 1)

    def test_intattr_with_min_invalid_default(self):
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=0, default=-1)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=1, default=0)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=2, default=1)

    def test_intattr_with_max_invalid_default(self):
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', max_value=0, default=1)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', max_value=1, default=2)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', max_value=2, default=3)

    def test_intattr_with_range_invalid_default(self):
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=0, max_value=0, default=1)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=1, max_value=0, default=2)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=1, max_value=0, default=2)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=1, max_value=1, default=2)

        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=2, max_value=0, default=3)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=0, max_value=2, default=3)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=2, max_value=2, default=3)
        self.assertRaises(ValueError, ztpserver.config.IntAttr,
            'test', min_value=2, max_value=3, default=4)

    def test_intattr_with_group(self):
        obj = ztpserver.config.IntAttr('test', group='test')
        self.assertIsInstance(obj, ztpserver.config.IntAttr)
        self.assertEqual(obj.group, 'test')

    def test_boolattr_defaults(self):
        obj = ztpserver.config.BoolAttr('test')
        self.assertIsInstance(obj, ztpserver.config.BoolAttr)
        self.assertEqual(repr(obj.type), "Boolean")

    def test_boolattr_with_default(self):
        obj = ztpserver.config.BoolAttr('test', default=True)
        self.assertIsInstance(obj, ztpserver.config.BoolAttr)
        self.assertTrue(obj.default)

    def test_boolattr_with_group(self):
        obj = ztpserver.config.BoolAttr('test', group='test')
        self.assertIsInstance(obj, ztpserver.config.BoolAttr)
        self.assertEqual(obj.group, 'test')

class TestGroup(unittest.TestCase):
    #pylint: disable=C0103

    class Config(object):
        def __init__(self):
            self.attributes = list()

        @classmethod
        def __get_attribute__(cls, name, group_name):
            return (name, group_name)

        def add_attribute(self, item, group_name):
            self.attributes.append((item, group_name))

        def __repr__(self):
            return 'Config'

    def setUp(self):
        self.config = self.Config()

    def tearDown(self):
        del self.config

    def test_group(self):
        obj = ztpserver.config.Group('test', self.config)
        self.assertIsInstance(obj, ztpserver.config.Group)
        self.assertEqual('test', obj.name)
        self.assertEqual('Config', repr(obj.config))

    def test_group_get_attr(self):
        obj = ztpserver.config.Group('test', self.config)
        self.assertEqual(obj.testattr, ('testattr', 'test'))
        self.assertEqual(obj['testattr'], ('testattr', 'test'))

    def test_group_add_attribute(self):
        obj = ztpserver.config.Group('test', self.config)
        obj.add_attribute('test')
        self.assertEqual(self.config.attributes[0], ('test', 'test'))


class TestConfig(unittest.TestCase):
    #pylint: disable=C0103

    Attr = collections.namedtuple('Attr', ['name', 'type', 'group', 'default'])
    CONF = """\n[default]\ntest = value\n[group]\ntest = value\n"""

    def setUp(self):
        self.config = ztpserver.config.Config()

    def tearDown(self):
        del self.config

    def test_config(self):
        self.assertIsInstance(self.config, ztpserver.config.Config)
        self.assertEqual(repr(self.config), 'Config')

    def test_config_add_attribute_valid(self):
        attr = self.Attr('test', str, None, None)
        self.config.add_attribute(attr)
        self.assertIsNone(getattr(self.config, 'test'))
        self.assertIn((None, 'test'), self.config.attributes)

    def test_config_add_attribute_default_value(self):
        attr = self.Attr('test', str, None, 'value')
        self.config.add_attribute(attr)
        self.assertEqual(self.config.test, 'value')

    def test_config_add_attribute_group_and_default_value(self):
        attr = self.Attr('test', str, 'group', 'value')
        self.config.add_attribute(attr)
        self.assertEqual(self.config.group.test, 'value')
        self.assertIsInstance(self.config.group, ztpserver.config.Group)

    def test_config_add_attribute_invalid(self):
        self.assertRaises(AttributeError, self.config.add_attribute, 'test')

    def test_config_add_attribute_duplicate(self):
        attr = self.Attr('test', str, None, None)
        self.config.add_attribute(attr)
        self.assertRaises(AttributeError, self.config.add_attribute, attr)

    def test_config_add_group(self):
        for grp in ['test', 100]:
            self.config.add_group(grp)
            grp = str(grp) if isinstance(grp, int) else grp
            self.assertIn(grp, self.config.groups)
            self.assertIsInstance(getattr(self.config, grp),
                                  ztpserver.config.Group)

    def test_config_set_value_attribute(self):
        attr = self.Attr('test', str, None, None)
        self.config.add_attribute(attr)
        self.config.set_value('test', 'value')
        self.assertEqual(self.config.test, 'value')

    def test_config_runtime(self):
        obj = ztpserver.config.runtime
        self.assertIsInstance(obj, ztpserver.config.Config)

    def test_config_runtime_clear_value(self):
        obj = ztpserver.config.runtime
        obj.set_value('interface', '1.1.1.1', 'server')
        self.assertEqual(obj.server.interface, '1.1.1.1')
        obj.clear_value('interface', 'server')
        self.assertEqual(obj.server.interface, '0.0.0.0')

    def test_config_runtime_set_value(self):
        obj = ztpserver.config.runtime
        obj.set_value('interface', '1.1.1.1', 'server')
        self.assertEqual(obj.server.interface, '1.1.1.1')

    def test_config_runtime_environ_env_set(self):
        os.environ['ZTPS_ENV'] = 'environ'
        obj = ztpserver.config.Config()
        attr = ztpserver.config.StrAttr(name='env',
                                        default='default',
                                        environ='ZTPS_ENV')
        obj.add_attribute(attr)
        self.assertEqual(obj.default.env, 'environ')

    def test_config_runtime_environ_env_notset(self):
        obj = ztpserver.config.Config()
        attr = ztpserver.config.StrAttr(name='env',
                                        default='default',
                                        environ='ZTPS_ENV')
        obj.add_attribute(attr)
        self.assertEqual(obj.default.env, 'default')

    def test_config_runtime_environ_env_set_no_default(self):
        os.environ['ZTPS_ENV'] = 'environ'
        obj = ztpserver.config.Config()
        attr = ztpserver.config.StrAttr(name='env',
                                        environ='ZTPS_ENV')
        obj.add_attribute(attr)
        self.assertEqual(obj.default.env, 'environ')

    def test_config_runtime_environ_env_notset_no_default(self):
        obj = ztpserver.config.Config()
        attr = ztpserver.config.StrAttr(name='env',
                                        environ='ZTPS_ENV')
        obj.add_attribute(attr)
        self.assertIsNone(obj.default.env)

    def test_load_config_file(self):
        filename = tempfile.NamedTemporaryFile(mode='w')
        filename.writelines(self.CONF)
        filename.flush()

        obj = ztpserver.config.runtime
        obj.add_attribute(ztpserver.config.StrAttr(name='test'))
        obj.add_attribute(ztpserver.config.StrAttr(name='test', group='group'))
        obj.read(filename.name)

        self.assertEqual(obj.default.test, 'value')
        self.assertEqual(obj.group.test, 'value')
        filename.close()


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
# pylint: disable=R0904,C0103
#
import unittest

from mock import patch

from ztpserver.serializers import SerializerError

from ztpserver.repository import FileObject, FileObjectError
from ztpserver.repository import Repository, RepositoryError
from ztpserver.repository import FileObjectNotFound

from server_test_lib import enable_logging, random_string

class FileObjectUnitTests(unittest.TestCase):

    @patch('ztpserver.serializers.load')
    def test_read_success(self, m_load):
        m_load.return_value = random_string()
        obj = FileObject(random_string())
        result = obj.read()
        self.assertEqual(m_load.return_value, result)

    @patch('ztpserver.serializers.load')
    def test_read_failure(self, m_load):
        m_load.side_effect = SerializerError
        obj = FileObject(random_string())
        self.assertRaises(FileObjectError, obj.read)

    @classmethod
    @patch('ztpserver.serializers.dump')
    def test_write_success(cls, _):
        obj = FileObjectError(random_string())
        obj = FileObject(random_string())
        obj.write(random_string())

    @patch('ztpserver.serializers.dump')
    def test_write_failure(self, m_dump):
        m_dump.side_effect = SerializerError
        obj = FileObject(random_string())
        self.assertRaises(FileObjectError, obj.write, random_string())


class RepositoryUnitTests(unittest.TestCase):

    @classmethod
    @patch('os.makedirs')
    def test_add_folder_success(cls, _):
        store = Repository(random_string())
        store.add_folder(random_string())

    @patch('os.makedirs')
    def test_add_folder_failure(self, m_makedirs):
        m_makedirs.side_effect = OSError
        store = Repository(random_string())
        self.assertRaises(RepositoryError, store.add_folder, random_string())

    @patch('ztpserver.repository.FileObject')
    @patch('os.chmod')
    def test_create_file_success(self, _, m_fileobj):
        store = Repository(random_string())
        store.add_file(random_string())
        self.assertFalse(m_fileobj.return_value.write.called)

    @patch('ztpserver.repository.FileObject')
    @patch('os.chmod')
    def test_create_file_with_contents_success(self, _,
                                               m_fileobj):
        store = Repository(random_string())
        store.add_file(random_string(), random_string())
        self.assertTrue(m_fileobj.return_value.write.called)
            
    @patch('ztpserver.repository.FileObject')
    def test_create_file_failure(self, m_fileobj):
        m_fileobj.return_value.write.side_effect = FileObjectError
        store = Repository(random_string())
        self.assertRaises(FileObjectError, store.add_file,
                          random_string(), random_string())

    @patch('os.path.exists')
    def test_exists_success(self, _):
        store = Repository(random_string())
        result = store.exists(random_string())
        self.assertTrue(result)

    @patch('os.path.exists')
    def test_exists_missing_file(self, m_exists):
        m_exists.return_value = False
        store = Repository(random_string())
        result = store.exists(random_string())
        self.assertFalse(result)

    @patch('os.path.exists')
    @patch('ztpserver.repository.FileObject')
    def test_get_file_success(self, m_fileobj, _):
        store = Repository(random_string())
        store.get_file(random_string())
        self.assertTrue(m_fileobj.called)

    @patch('os.path.exists')
    @patch('ztpserver.repository.FileObject')
    def test_get_file_failure(self, m_fileobj, m_exists):
        m_exists.return_value = False
        store = Repository(random_string())
        self.assertRaises(FileObjectNotFound, store.get_file, random_string())
        self.assertFalse(m_fileobj.called)

    @patch('os.remove')
    def test_delete_file_success(self, m_remove):
        store = Repository(random_string())
        store.delete_file(random_string())
        self.assertTrue(m_remove.called)

    @patch('os.remove')
    def test_delete_file_failure(self, m_remove):
        m_remove.side_effect = OSError
        store = Repository(random_string())
        self.assertRaises(RepositoryError, store.delete_file, random_string())


if __name__ == '__main__':
    enable_logging()
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
# pylint: disable=C0103
#
import json
import os
import logging
import random
import shutil
import string        #pylint: disable=W0402

import yaml

from mock import Mock
from ztpserver.app import enable_handler_console

WORKINGDIR = '/tmp/ztpserver'

log = logging.getLogger('ztpserver')
log.setLevel(logging.DEBUG)
log.addHandler(logging.NullHandler())

def enable_logging():
    enable_handler_console()

def random_string():
    return ''.join(random.choice(string.ascii_uppercase + string.digits)
                   for _ in range(random.randint(3, 20)))

def random_json(keys=None):
    data = dict()
    if keys:
        for key in keys:
            data[key] = random_string()
    else:
        for _ in range(0, 5):
            data[random_string()] = random_string()
    return json.dumps(data)

def remove_all():
    if os.path.exists(WORKINGDIR):
        shutil.rmtree(WORKINGDIR)

def add_folder(name=None):
    if not name:
        name = random_string()
    filepath = os.path.join(WORKINGDIR, name)
    if os.path.exists(filepath):
        shutil.rmtree(filepath)
    os.makedirs(filepath)
    return filepath

def write_file(contents, filename=None, mode='w'):
    if not filename:
        filename = random_string()
    if not os.path.exists(WORKINGDIR):
        os.makedirs(WORKINGDIR)
    filepath = os.path.join(WORKINGDIR, filename)
    open(filepath, mode).write(contents)
    return filepath

def ztp_headers():
    return {
        'X-Arista-Systemmac': random_string(),
        'X-Arista-Serialnum': random_string(),
        'X-Arista-Architecture': random_string(),
        'X-Arista-Modelname': random_string(),
        'X-Arista-Softwareversion': random_string()
    }

class SerializerMixin(object):
    def as_dict(self):
        raise NotImplementedError

    def as_yaml(self):
        return yaml.dump(self.as_dict())

    def as_json(self):
        return json.dumps(self.as_dict())

class Definition(SerializerMixin):

    def __init__(self, **kwargs):
        self.name = kwargs.get('name')
        self.attributes = kwargs.get('attributes', dict())
        self.actions = kwargs.get('actions', list())

    def add_attribute(self, key, value):
        self.attributes[key] = value

    def add_action(self, **kwargs):
        action = dict()
        action['name'] = kwargs.get('name', 'test action')
        action['action'] = kwargs.get('action', 'test_action')
        action['attributes'] = kwargs.get('attributes', dict())
        action['always_execute'] = kwargs.get('always_execute', dict())
        self.actions.append(action)

    def as_dict(self):
        return dict(name=self.name,
                    attributes=self.attributes,
                    actions=self.actions)

def create_definition():
    return Definition()

class Attributes(SerializerMixin):

    def __init__(self, **kwargs):
        self.attributes = kwargs.get('attributes', dict())

    def add_attribute(self, key, value):
        self.attributes[key] = value

    def add_attributes(self, attributes):
        assert isinstance(attributes, dict)
        for key, value in attributes.items():
            self.add_attribute(key, value)

    def as_dict(self):
        return self.attributes

def create_attributes():
    return Attributes()

class NodeDict(SerializerMixin):

    def __init__(self, **kwargs):
        self.serialnumber = kwargs.get('serialnumber', random_string())
        self.model = kwargs.get('model', random_string())
        self.version = kwargs.get('version', random_string())
        self.systemmac = kwargs.get('systemmac', random_string())
        self.neighbors = kwargs.get('neighbors', dict())

    def add_random_neighbor(self, interface):
        neighbor = dict(device=random_string(), port=random_string())
        self.add_neighbor(interface, neighbor)

    def add_neighbor(self, interface, peer):
        if interface not in self.neighbors:
            self.neighbors[interface] = list()
        self.neighbors[interface].append(peer)

    def add_neighbors(self, neighbors):
        assert isinstance(neighbors, dict)
        for key, value in neighbors.items():
            self.add_neighbor(key, value)

    def as_dict(self):
        return dict(systemmac=self.systemmac,
                    serialnumber=self.serialnumber,
                    model=self.model,
                    version=self.version,
                    neighbors=self.neighbors)

def create_node():
    return NodeDict()

def mock_match(node=None, definition=None,
               variables=None,
               name=None,
               interfaces=None):
    if not definition:
        definition = random_string()
    if not variables:
        variables = {}
    if not interfaces:
        interfaces = []
    if not name:
        name = random_string()

    match = Mock()
    match.serialize.return_value = {
        'node': node,
        'definition': definition,
        'variables': variables, 
        'name': name, 
        'interfaces': interfaces}

    return match

class BootstrapConf(SerializerMixin):
    def __init__(self, **kwargs):
        self.logging = kwargs.get('logging', list())
        self.xmpp = kwargs.get('xmpp', dict())

    def add_logging(self, entry):
        self.logging.append(entry)

    def as_dict(self):
        return dict(logging=self.logging, xmpp=self.xmpp)

def create_bootstrap_conf():
    return BootstrapConf()

class Pattern(SerializerMixin):

    def __init__(self, **kwargs):

        self.name = kwargs.get('name', random_string())
        self.node = kwargs.get('node')
        self.definition = kwargs.get('definition', random_string())
        self.interfaces = list()

    def add_interface(self, local_intf, remote_device, remote_intf):
        self.interfaces.append({local_intf: {'device': remote_device,
                                             'port': remote_intf}})

    def add_random_interface(self, count=1):
        for i in range(0, count):       # pylint: disable=W0612
            while True:
                intf = 'Ethernet%d' % random.randint(1, 64)
                if intf not in self.interfaces:
                    self.add_interface(intf, random_string(), random_string())
                    break

    def as_dict(self):
        return dict(name=self.name,
                    node=self.node,
                    definition=self.definition,
                    interfaces=self.interfaces)

def create_pattern():
    return Pattern()

class NeighborDb(SerializerMixin):

    def __init__(self, **kwargs):
        self.name = kwargs.get('name', random_string())
        self.variables = kwargs.get('variables', dict())
        self.patterns = kwargs.get('patterns', list())

    def get_patterns(self):
        return [x.as_dict() for x in self.patterns]

    def add_pattern(self, pattern=None):
        if not pattern:
            pattern = Pattern()
            pattern = pattern.as_dict()
        self.patterns.append(pattern)

    def add_variable(self, key, value):
        self.variables[key] = value

    def as_dict(self):
        return dict(name=self.name,
                    variables=self.variables,
                    patterns=[x.as_dict() for x in self.patterns])

def create_neighbordb():
    return NeighborDb()





#!/usr/bin/env python
#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#  - Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#  - Neither the name of Arista Networks nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#pylint: disable=R0904,F0401,W0232,E1101

import os
import os.path
import unittest
import shutil
import sys

sys.path.append('test/client')

from client_test_lib import Bootstrap, ActionFailureTest
from client_test_lib import file_log, get_action, random_string
from client_test_lib import startup_config_action, remove_file
from client_test_lib import raise_exception

class FailureTest(ActionFailureTest):

    def test_missing_url(self):
        self.basic_test('install_extension',
                        'Missing attribute(\'url\')')

    def test_url_failure(self):
        self.basic_test('install_extension',
                        'Unable to retrieve extension from URL',
                        attributes={'url' :
                                    random_string()})


class SuccessTest(unittest.TestCase):

    def test_success(self):
        bootstrap = Bootstrap(ztps_default_config=True)

        extension = random_string()
        url = extension

        extension_force = random_string()
        url_force = 'http://%s/%s' % (bootstrap.server, extension_force)

        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'},
                     {'action' : 'test_action',
                      'attributes' : {'url' : url}},
                     {'action' : 'test_action_force',
                      'attributes' :
                      {'url' : url_force,
                       'force' : True}}
                     ])

        bootstrap.ztps.set_action_response(
            'startup_config_action', startup_config_action())

        extensions_dir = '/tmp/extensions'
        boot_extensions = '/tmp/boot-extensions'
        action = get_action('install_extension')
        action = action.replace('/mnt/flash/.extensions',
                                extensions_dir)
        action = action.replace('/mnt/flash/boot-extensions',
                                boot_extensions)

        bootstrap.ztps.set_action_response('test_action',
                                           action)
        contents = random_string()
        bootstrap.ztps.set_file_response(extension, contents)

        bootstrap.ztps.set_action_response('test_action_force',
                                           action)
        contents_force = random_string()
        bootstrap.ztps.set_file_response(extension_force, contents_force)

        bootstrap.start_test()

        try:
            ext_filename = '%s/%s' % (extensions_dir, extension)
            self.failUnless(os.path.isfile(ext_filename))
            self.failUnless([contents] ==
                            file_log(ext_filename))
            self.failUnless(extension in file_log(boot_extensions))

            ext_filename_force = '%s/%s' % (extensions_dir, extension_force)
            self.failUnless(os.path.isfile(ext_filename_force))

            self.failUnless([contents_force] ==
                            file_log(ext_filename_force))
            self.failUnless('%s force' % extension_force in
                            file_log(boot_extensions))

            self.failUnless(bootstrap.success())
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            shutil.rmtree(extensions_dir)
            remove_file(boot_extensions)
            bootstrap.end_test()

    def test_url_success(self):
        bootstrap = Bootstrap(ztps_default_config=True)

        extension = random_string()
        url = 'http://%s/%s' % (bootstrap.server, extension)

        extension_force = random_string()
        url_force = 'http://%s/%s' % (bootstrap.server, extension_force)

        bootstrap.ztps.set_definition_response(
            actions=[{'action' : 'startup_config_action'},
                     {'action' : 'test_action',
                      'attributes' : {'url' : url}},
                     {'action' : 'test_action_force',
                      'attributes' :
                      {'url' : url_force,
                       'force' : True}}
                     ])

        bootstrap.ztps.set_action_response(
            'startup_config_action', startup_config_action())

        extensions_dir = '/tmp/extensions'
        boot_extensions = '/tmp/boot-extensions'
        action = get_action('install_extension')
        action = action.replace('/mnt/flash/.extensions',
                                extensions_dir)
        action = action.replace('/mnt/flash/boot-extensions',
                                boot_extensions)

        bootstrap.ztps.set_action_response('test_action',
                                           action)
        contents = random_string()
        bootstrap.ztps.set_file_response(extension, contents)

        bootstrap.ztps.set_action_response('test_action_force',
                                           action)
        contents_force = random_string()
        bootstrap.ztps.set_file_response(extension_force, contents_force)

        bootstrap.start_test()

        try:
            ext_filename = '%s/%s' % (extensions_dir, extension)
            self.failUnless(os.path.isfile(ext_filename))
            self.failUnless([contents] ==
                            file_log(ext_filename))
            self.failUnless(extension in file_log(boot_extensions))

            ext_filename_force = '%s/%s' % (extensions_dir, extension_force)
            self.failUnless(os.path.isfile(ext_filename_force))

            self.failUnless([contents_force] ==
                            file_log(ext_filename_force))
            self.failUnless('%s force' % extension_force in
                            file_log(boot_extensions))

            self.failUnless(bootstrap.success())
        except AssertionError as assertion:
            print 'Output: %s' % bootstrap.output
            print 'Error: %s' % bootstrap.error
            raise_exception(assertion)
        finally:
            shutil.rmtree(extensions_dir)
            remove_file(boot_extensions)
            bootstrap.end_test()

if __name__ == '__main__':
    unittest.main()
